pub trait Not<T> { }Expand description
Not<T> is implemented for U if and only if U is not equal to T
Dyn Compatibility
This trait is dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".
pub trait Not<T> { }Not<T> is implemented for U if and only if U is not equal to T
This trait is dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".